@@ -124,10 +124,12 @@ class HA_OmnikDataLogger(hass.Hass):
         self.rt = RepeatedJob(c, self.datalogger, hass_api=self)
 
     def terminate(self):
-        self.datalogger.terminate()
-        hybridlogger.ha_log(logger, self, "INFO", "DSMR Daemon was stopped.")
+        hybridlogger.ha_log(logger, self, "INFO", "Stopping omnikdatalogger daemon...")
         self.rt.stop()
         hybridlogger.ha_log(logger, self, "INFO", "Daemon was stopped.")
+        hybridlogger.ha_log(logger, self, "INFO", "Stopping DSMR Daemon...")
+        self.datalogger.terminate()
+        hybridlogger.ha_log(logger, self, "INFO", "DSMR Daemon was stopped.")
 
 
 # Initialization from the commandline
@@ -138,6 +140,7 @@ def main(c: ha_ConfigParser, hass_api=None):
     # Enabled debugging if the flag is set
     if c.get('default', 'debug', False):
         logger.setLevel(logging.DEBUG)
+
     set_data_config_path(c)
     datalogger = DataLogger(c, hass_api=hass_api)
     if c.has_option('default', 'interval'):
